;**********************************************************************
;   This file is a basic code template for object module code         *
;   generation on the PICmicro PIC16F684. This file contains the      *
;   basic code building blocks to build upon.  As a project minimum   *
;   the 16F684.lkr file will also be required for this file to        *
;   correctly build. The .lkr files are located in the MPLAB          *
;   directory.                                                        *
;                                                                     *
;   If interrupts are not used all code presented between the         *
;   code section "INT_VECTOR and code section "MAIN" can be removed.  *
;   In addition the variable assignments for 'w_temp' and             *
;   'status_temp' can be removed.                                     *
;                                                                     *
;   If interrupts are used, as in this template file, the 16F684.lkr  *
;   file will need to be modified as follows: Remove the lines        *
;     CODEPAGE   NAME=vectors  START=0x0      END=0x4      PROTECTED  *
;   and                                                               *
;     SECTION    NAME=STARTUP  ROM=vectors                            *
;   and change the start address of the page0 section from 0x5 to 0x0 *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler and linker (Document DS33014).          *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    piccoz.asm                                        *
;    Date: 24/09/2007                                                 *
;    File Version: 4.1 version avion                                  *
;    modif du 21/02/07 : latchage depuis l'interruption               *
;  + ajout d'un allumage de la led si metteur coup ou pas de signal *
;  modif du 04/04/07 : suppression coupure des gaz de 12us            *
;	 modif du 09/03/07 : changement de l'chelle des dures           *
;	 d'impulsions pour viter l'usage de l'ATV                        *
;    modif du 24/09/07 : rvision du systme de masques, lors du      *
;    passage  zro on n'a plus de cas particulier. Les sorties sont  *
;    actionnes sans retard garantissant un vrai 100% possible.       *
;    Seules 17 positions sont possibles par axe 0,16,32,...240,255    *
;    modif du 17/10/07 Frquence PWM = 2 kHz pour toutes les voies,   *
;    limite physique                                                  *
;    modif du 28/11/07 : raffectation de toutes les entres/sorties  *
;    pour tre compatible avec le typon 4.1 et le 16F616 boitier ML   *
;    + conversion en 1 voie gaz + 3 voies actuateurs (avion)          *
;	 Author: J. Jeambrun                                              *
;    Company: JJ STAR SYSTEMS                                         *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                    16F616.lkr                                       *
;**********************************************************************
;  RC0 : sortie PWM voie 1 gaz, patte 9                               *
;  RA2 : entre signal PPM patte 10                                   *
;  RC1 : sortie PWM voie 4 derive patte 8 si positif                  *
;  RC2 : sortie PWM voie 4 derive patte 7 si ngatif                  *
;  RC3 : sortie LED patte 6                                           *
;  RC5 : sortie PWM voie 3 profondeur patte 4 si positif              *
;  RC4 : sortie PWM voie 3 profondeur patte 5 si ngatif              *
;  RA5 : sortie PWM voie 2 ailerons patte 1 si positif                *
;  RA4 : sortie PWM voie 2 ailerons patte 2 si ngatif                *
;**********************************************************************

	list      p=16f616           	; list directive to define processor
	#include <p16F616.inc>        	; processor specific variable definitions

;	errorlevel  -302              	; suppress message 302 from list file
;
	__CONFIG    _PWRTE_ON & _WDT_OFF & _MCLRE_OFF & _CP_OFF &_INTRC_OSC_NOCLKOUT

;******MACROS**********************************************************
; changement de banques
BANK0		macro
			BCF	STATUS,RP0
			BCF	STATUS,RP1
			endm
BANK1		macro
			BSF	STATUS,RP0
			BCF	STATUS,RP1
			endm
;
;
;***** VARIABLE DEFINITIONS (examples)
RET0_A		EQU		0x20			; retards provisoires (calculs mais non valids) de 0  255
RET1_A		EQU		0x21
RET2_A		EQU		0x22
RET3_A		EQU		0x23
RET4_A		EQU		0x24
RET5_A		EQU		0x25			; pour les besoins du calcul, reste  zro
;
MASK0A_A	EQU		0x26			; masque qui sera crit sur le port A au RAZ PWM
MASK1A_A	EQU		0x27			; masque qui met la sortie PWM N1  l'tat haut (port A)
MASK2A_A	EQU		0x28			; masque qui met la sortie PWM N2  l'tat haut (port A)
MASK3A_A	EQU		0x29			; masque qui met la sortie PWM N3  l'tat haut (port A)		
MASK4A_A	EQU		0x2A
;
MASK0C_A	EQU		0x2B			; masque qui sera crit sur le port C au RAZ PWM
MASK1C_A	EQU		0x2C			; masque qui met la sortie PWM N1  l'tat haut
MASK2C_A	EQU		0x2D
MASK3C_A	EQU		0x2E
MASK4C_A	EQU		0x2F
;
RET0_B		EQU		0x30			; retards provisoires (calculs mais non valids) de 0  255
RET1_B		EQU		0x31
RET2_B		EQU		0x32
RET3_B		EQU		0x33
RET4_B		EQU		0x34
RET5_B		EQU		0x35			; pour les besoins du calcul, reste  zro
;
MASK0A_B	EQU		0x36			; adresse rserve inutilise
MASK1A_B	EQU		0x37			; masque qui met la sortie PWM N1  l'tat haut (port A)
MASK2A_B	EQU		0x38			; masque qui met la sortie PWM N2  l'tat haut (port A)
MASK3A_B	EQU		0x39			; masque qui met la sortie PWM N3  l'tat haut (port A)		
MASK4A_B	EQU		0x3A
;
MASK0C_B	EQU		0x3B			; masque qui sera crit sur le port C au RAZ PWM
MASK1C_B	EQU		0x3C			; masque qui met la sortie PWM N1  l'tat haut
MASK2C_B	EQU		0x3D
MASK3C_B	EQU		0x3E
MASK4C_B	EQU		0x3F
;
TPS_ON1		EQU 	0x40			; temps  1 de la sortie voie 1 compris entre 0 et 254
TPS_ON2		EQU		0x41
TPS_ON3		EQU		0x42
TPS_ON4		EQU		0x43
TPS_ON5		EQU		0x44			; virtuel, utilis pour le calcul, doit tre  zro en permanence
;
CPT_IMP		EQU		0x45			; compte les impulsions des voies 1  8
FLAGS		EQU		0x46			; flags d'usage gnral
;
DUR_IMP1	EQU		0x47			;dure impulsion voie 1 comprise entre 17 < 80 < 144 dcimal
DUR_IMP2	EQU		0x48			;dure impulsion voie 2 comprise entre 16 < 80 < 144 dcimal
DUR_IMP3	EQU		0x49			;dure impulsion voie 3 comprise entre 16 < 80 < 144 dcimal
DUR_IMP4	EQU		0x4A			;dure impulsion voie 4 comprise entre 17 < 80 < 144 dcimal
DUR_IMP5	EQU		0x4B
DUR_IMP6	EQU		0x4C
DUR_IMP7	EQU		0x4D
DUR_IMP8	EQU		0x4E
;
DUR_IMPX	EQU		0x4F			; variable de stockage de DUR_IMP pendant le tri
TPS_ONX		EQU		0x50			; idem pour TPS_ON
MASKX		EQU		0x51			; idem pour MASK
ADR			EQU		0x52				; stocke une adresse pour les op en adressage indirect
;
CPT_TI		EQU		0x53			; compteur de trames invalides
CPT_DEM		EQU		0x54			; anticoupe doigts : gre le temps o les gaz doivent tre  zero avant de latcher RET(i)
VARI		EQU		0x55			; compteur gnral pour les boucles, attention utilis en interruption !
VARK		EQU		0x56			; compteur gnral pour les boucles
VARN		EQU		0x57			; compteur gnral pour les boucles
w_temp		EQU		0x58			; variable used for context saving
status_temp	EQU     0x59			; variable used for context saving
CPT_TIP		EQU		0x5A			; compteur de trames invalides affiches par LED = cpt trames invalides provisoire
CPT_ON		EQU		0x5B			; dure d'allumage de la diode en nb de trames de 20ms
CPT_OFF		EQU		0x5C			; dure diode teinte lors des clignotements en nb de trames
FSR_TEMP	EQU		0x5D			; sauvegarde contexte lors de l'interruption
PCL_TEMP	EQU		0x5F			; sauvegarde contexte lors de l'interruption
VARZ		EQU		0x60			; variable gnrale rserve  l'interruption
SUMRET		EQU		0x61			; compteur somme des retards lus, permet de ne pas faire
;									frtiller les voies  zro (gre en direct un rapport cyclique nul)
ADR_FIN		EQU		0x62			; adresse de fin utilise dans les balayages
ADR_RET0	EQU		0x63			; adresse contenant Retard 0 actif dans l'interruption, l'autre tant le provisoire
;
; example of using Shared Uninitialized Data Section
;INT_VAR		UDATA_SHR	0x71   
;w_temp		RES     1		; variable used for context saving 
;status_temp	RES     1		; variable used for context saving
;
; dfinition des constantes
V2P			EQU		5	; N bit voie 2 si positive = sortie RA5		
V3P			EQU		5	; N bit voie 3 si positive = sortie RC5
V4P			EQU		1	; N bit voie 4 si positive = sortie RC1
V2M			EQU		4	; N bit voie 2 si ngative = sortie RA4		
V3M			EQU		4	; N bit voie 3 si ngative = sortie RC4
V4M			EQU		2	; N bit voie 4 si ngative = sortie RC2
LED			EQU		3	; sortie LED = RC3, 0 si allume
ENTIMP		EQU		2	; entre impulsion rception IR
w			EQU		0
f			EQU		1
FNT			EQU		0	; utilis dans FLAGS, Flag New Trame ie une trame neuve est dispo
GAZMAX		EQU		1	; utilis dans FLAGS, Flag gaz  fond
GAZ			EQU		0	; portC, RC0
;
;
;**********************************************************************
RESET_VECTOR	ORG 	0x000		; processor reset vector
				goto    main            ; go to beginning of program


INT_VECTOR	ORG		0x004		; interrupt vector location
			movwf   w_temp          ; save off current W register contents
			movf	STATUS,w        ; move status register into W register
			movwf	status_temp     ; save off contents of STATUS register
			MOVF	FSR,w
			MOVWF	FSR_TEMP
;			MOVF	PCLATH,w
;			MOVWF	PCL_TEMP
			BANK0
; est-on  la fin du cycle PWM ? (ie somme des retards lus = 255), attention multiples de 4 !
			INCF	SUMRET,w
			BTFSS	STATUS,Z		; SUMRET = 255 ?
			GOTO	D8LBL2			; si non on applique les masques
; si oui, inutile de se fatiguer, toutes les voies suivantes sont  zro, on rinitialise le gnrateur PWM
			BTFSC	FLAGS,FNT		; oui vari=5
			CALL	LATCH			; si nouvelle trame dispo on latche avant de tout couper !
			CLRF	VARI
			CLRF	SUMRET
;
			MOVF	ADR_RET0,w		; RAZ sorties RA 
			ADDLW	0x06
			MOVWF	FSR				; pointer MASK0A
			MOVF	INDF,w
			MOVWF	PORTA			; actionner portA
;			
			MOVLW	0x05
			ADDWF	FSR,f			; pointer MASK0C
			MOVF	INDF,w
			MOVWF	PORTC			; actionner portC
			GOTO	D8LBL3			; sauter l'application des masques
D8LBL2		MOVF	ADR_RET0,w
			ADDLW	0x06			; diffrence entre RET0 et MASK0A
			ADDWF	VARI,w
			MOVWF	FSR				; adresse de MASK0A + VARI dans FSR
			MOVF	INDF,w			; charger MASK(i)A
			IORWF	PORTA,f			; appliquer masque en cours sur port A
			MOVLW	0x05			; dcalage entre MASK(i)A et MASK(i)C
			ADDWF	FSR,f
			MOVF	INDF,w
			IORWF	PORTC,f			; appliquer masque en cours sur port C
D8LBL3		MOVF	ADR_RET0,w		; adresse de RET0
			ADDWF	VARI,w			; adresse de RET(i) dans w
			MOVWF	FSR				; RET(i) dans FSR
			MOVF	INDF,w			; charger RET(i) dans w
			ADDWF	SUMRET,f		; sommer les retards lus
;			MOVF	INDF,w			; charger RET(i)
			BANK1
			MOVWF	PR2
			BANK0
			INCF	VARI,f
			BCF		PIR1,TMR2IF		; autoriser interrupt suivante
			BSF		INTCON,GIE
; restauration temporaires
;			MOVF	PCL_TEMP,w
;			MOVWF	PCLATH
			MOVF	FSR_TEMP,w
			MOVWF	FSR
			movf    status_temp,W   ; retrieve copy of STATUS register
			movwf	STATUS          ; restore pre-isr STATUS register contents
			swapf   w_temp,F
			swapf   w_temp,W        ; restore pre-isr W register contents
			retfie                  ; return from interrupt
;
;
main
; initialisations gnrales
			BANK1
			CLRF	OSCTUNE			; calibration d'usine pour l'oscillateur
			CLRF	ANSEL			; tout E/S numriques
			BCF		ADCON0,ADON		; pas de convertisseur A/D
			MOVLW	b'00000011'
			MOVWF	OPTION_REG		; pull-up enable portA, T0clock source internal, PS  TMR0, PS=1/16
			MOVLW	b'11000000'
			MOVWF	INTCON			; GIenable, Periph int enabled, TMR0 int disabled, PortA int disabled
			MOVLW	b'00110000'		
			MOVWF	WPUA			; RA4 et RA5 pull-up enable individuel
			MOVLW	b'00001111'
			MOVWF	TRISA			; RA2:PPM in, RA4 et RA5 sortie voie 2
			MOVLW	b'00000000'
			MOVWF	TRISC			; RC tout en sortie
			CLRF	VRCON			; pas de rfrence de tension
;			
			BANK0
			BCF		ADCON0,ADON		; pas de convertisseur A/D
			CLRF	PORTC			; toutes les sorties  zero
			CLRF	PIR1			; interrupt flags  zro
;			MOVLW	b'00000110'
;			MOVWF	T2CON			; TMR2 ON, PS TMR2 = 1/16 (priode PWM = 0.5x16x256=2048s soit 500Hz)
			MOVLW	b'00000101'
			MOVWF	T2CON			; TMR2 ON, PS TMR2 = 1/4 (priode PWM = 0.5x4x256=512s soit 2000Hz)
			MOVLW	b'00010001'
			MOVWF	T1CON			; TMR1 ON, PS TMR1 = 1/2 (priode dfaut rception = 0.5x2x256*156*50=2s)
			BCF		CM1CON0,C1ON	; pas de comparateur1
			BCF		CM2CON0,C2ON	; pas de comparateur2
			CLRF	ADCON0			; pas de CAN
			CLRF	CCP1CON			; pas de module CCP
			BSF		PORTC,LED		; teindre la led
; initialisation des variables
			MOVLW	.100
			MOVWF	CPT_DEM			; dure d'attente gaz  zro avant latchage
			BSF		FLAGS,FNT		; on autorise  utiliser les valeurs de retards (premier passage)
HOT_RESET	CLRF	MASK0C_A
			BSF		MASK0C_A,LED	; teindre la led
			CLRF	MASK0C_B
			BSF		MASK0C_B,LED	; teindre la led
			MOVLW	0x20
			MOVWF	ADR_RET0
			CLRF	CPT_TIP			; trames invalides provisoires
			CLRF	CPT_TI
			MOVLW	.20
			MOVWF	CPT_ON			; dure d'allumage de la diode en nb de trames de 20ms 
			CLRF	TPS_ON5
			MOVLW	0xFF			; mise au neutre de toutes les sorties
			MOVWF	RET0_A
			CLRF	RET1_A
			CLRF	RET2_A
			CLRF	RET3_A
			CLRF	RET4_A
			CLRF	RET5_A
			CLRF	MASK0A_A
			CLRF	MASK1A_A
			CLRF	MASK2A_A
			CLRF	MASK3A_A
			CLRF	MASK4A_A
;
			CLRF	MASK1C_A
			CLRF	MASK2C_A
			CLRF	MASK3C_A
			CLRF	MASK4C_A
			MOVLW	0xFF			; mise au neutre de toutes les sorties
			MOVWF	RET0_B
			CLRF	RET1_B
			CLRF	RET2_B
			CLRF	RET3_B
			CLRF	RET4_B
			CLRF	RET5_B
			CLRF	MASK0A_B
			CLRF	MASK1A_B
			CLRF	MASK2A_B
			CLRF	MASK3A_B
			CLRF	MASK4A_B
;
			CLRF	MASK1C_B
			CLRF	MASK2C_B
			CLRF	MASK3C_B
			CLRF	MASK4C_B
; initialisation du gnrateur PWM
			MOVLW	0x05			; permet le retour  zro de VARI dans interrupt
			MOVWF	VARI
			MOVLW	0xFF
			MOVWF	SUMRET			; idem
			BCF		PIR1,TMR2IF
			BANK1
			MOVLW	0xFF
			MOVWF	PR2				; on initalise PR2
			MOVLW	b'00000010'
			MOVWF	PIE1			; TMR2 int enable
			BANK0
; initialisation scurit en cas d'absence de signal
			MOVLW	.99
			MOVWF	TMR1H
			CLRF	TMR1L
;
; attente d'une pause de 8.5 ms mini sur signal rception
ATTPAUSE	BCF		INTCON, T0IF
			BANK1
			MOVLW	b'00000110'
			MOVWF	OPTION_REG	; PS TMR0 pour 8 ms : 1/128
			BANK0
LBL1		BTFSS	PORTA,ENTIMP
			GOTO	LBL1			; attendre front montant
			MOVLW	.146
			MOVWF	TMR0			; charger TMR0 pour 7.0 ms = (255-146)x128x0.5
LBL2		BTFSS	PORTA,ENTIMP	; attendre front descendant
			GOTO	LBL2A
			BTFSS	PIR1,TMR1IF
			GOTO	LBL2
			CALL	DEB_T1			; appeler sub qui gere le dfaut dbordement TMR1
			GOTO	LBL2			
LBL2A		BTFSS	INTCON,T0IF
			GOTO	ATTPAUSE		; si la pause tait trop courte, ce n'tait pas celle de synchro
NEXTTRAM							; la pause tait assez longue, attente premire impulsion
			BANK1
			MOVLW	b'00000011'
			MOVWF	OPTION_REG		; PS TMR0 = 1/16
			BANK0
			CLRF	CPT_IMP
			MOVLW	.99
			MOVWF	TMR1H			; Rechargement tmr1 car tout est normal
			CLRF	TMR1L			; idem
FM1			CALL	ATT_FM			; attendre le 1er front montant
ACQIMP		BCF		INTCON,T0IF		; rinitialiser TMR0  chaque impulsion		
			MOVLW	.148
			MOVWF	TMR0
LBL4		CALL	ATT_FM			; attendre les fronts montants suivants
;			BTFSC	INTCON,T0IF
;			GOTO	LBL4A			; tout est ok
;			INCF	CPT_TI,f		; si parasite, attendre un front montant "probable" (<1.5 ms)
;			GOTO	ATTPAUSE
LBL4A		BCF		INTCON,T0IF
			INCF	CPT_IMP,f		; incrmenter n impulsion  chaque front montant dans les temps
; acquisition et stockage de la valeur de dure d'impulsion	par adressage indirect
			MOVLW	0x46			; adresse de DUR_IMP1 - 1
			MOVWF	FSR
			MOVF	CPT_IMP,w
			ADDWF	FSR,f			;0x46 + CPT_IMP ? FSR
			MOVF	TMR0,w			; nouvelle dure d'impulsion dans W
			MOVWF	INDF
;			ADDWF	INDF,f			; ancienne + nouvelle dure dans DUR_IMP(CPT_IMP)
;			RRF		INDF,f			; somme divise par 2 (through carry) pour lissage)
;
			MOVF	CPT_IMP,w
			SUBLW	0x08
			BTFSS	STATUS,Z		; CPT_IMP = 8 ?
			GOTO	ACQIMP			; si non acqurir impulsion suivante
; si oui, on traite les valeurs
			BANK1
			MOVLW	b'00000110'
			MOVWF	OPTION_REG		; PS TMR0  1/128
			BANK0
			MOVLW	.146			; tait .146 puis .155
			MOVWF	TMR0
			BCF		INTCON,T0IF		; vrifier que c'est ici
; gestion de l'allumage de la LED : 1 flash = 1 erreur
			MOVF	CPT_TI,w
			BTFSS	STATUS,Z		; compteur trames invalides = 0? (y a-t-il qq chose  afficher ?)
			GOTO	D1LBL4			; si dfaut prsent, afficher dfaut
			BSF		MASK0C_A,LED	; si pas de dfaut fin d'affichage
			BSF		MASK0C_B,LED
			GOTO	FINAFF
D1LBL4		MOVF	CPT_TIP,w
			BTFSS	STATUS,Z		; le train de flash en cours est-il termin ?
			GOTO	D1LBL1			; si CPT_TIP diffrent de zro
			MOVF	CPT_TI,w		; si CPT_TIP = 0,
			MOVWF	CPT_TIP		; on le recharge
			MOVLW	.20
			MOVWF	CPT_OFF			; et on programme une dure teinte
			GOTO	FINAFF
D1LBL1		MOVF	CPT_OFF,w
			BTFSC	STATUS,Z
			GOTO	D1LBL2			; si CPT_OFF = 0
			BSF		MASK0C_A,LED	; si CPT_OFF ? 0 teindre led o la garder teinte
			BSF		MASK0C_B,LED
			DECF	CPT_OFF,f
			GOTO	FINAFF
D1LBL2		MOVF	CPT_ON,w
			BTFSC	STATUS,Z
			GOTO	D1LBL3
			DECF	CPT_ON,f		; si CPT_ON diffrent de 0 on dcompte
			BCF		MASK0C_A,LED	; et on allume ou maintient allum
			BCF		MASK0C_B,LED
			GOTO	FINAFF
D1LBL3		MOVLW	.15				; si CPT_ON = 0, on le recharge (15x20ms)
			MOVWF	CPT_ON
			MOVLW	.20				; ainsi que CPT_OFF (20x20 ms)
			MOVWF	CPT_OFF
			DECF	CPT_TIP,f		; le flash est achev
			BSF		MASK0C_A,LED	; teindre led
			BSF		MASK0C_B,LED
FINAFF
; fin de l'affichage des dfauts par led
; teste si DUR_IMP(vark) > 163, si oui il y a un problme on retourne attendre la trame suivante
;
; DEBUT DE BORNAGE des valeurs de dures d'impulsions**************************************DETAIL 2
			MOVLW	.4
			MOVWF	VARK			; attention ne pas utiliser VARI cause interruption
			MOVLW	0x47
			MOVWF	FSR				; adresse de DUR_IMP1
D2LBL2		MOVF	INDF,w
; teste si DUR_IMP(vark) > 163, si oui il y a un problme on retourne attendre la trame suivante
			SUBLW	.163			; 163 - w ? w
			BTFSC	STATUS,C		; DUR_IMP(K) > 163 ?
			GOTO	D2LBL4			; non on ne fait rien
			INCF	CPT_TI,f		; si oui, on signale le dfaut
			GOTO	ATTPAUSE		; et on retourne attendre une trame OK
; fin de test de dure impulsion anormalement longue
; dbut bornage classique
D2LBL4		MOVF	INDF,w
			SUBLW	.131			; 131 - w ? w
			BTFSC	STATUS,C		; DUR_IMP(K) > 131 ?
			GOTO	D2LBL1			; non on ne fait rien
			MOVLW	.131
			MOVWF	INDF			; oui on borne DUR_IMP(K)  .131
			GOTO	D2LBL3
D2LBL1		MOVF	INDF,w
			SUBLW	.29				; 29 - w ? w
			BTFSS	STATUS,C
			GOTO	D2LBL3			; si DUR_IMP(K) > 29 alors rien
			MOVLW	.29
			MOVWF	INDF			; si DUR_IMP(K) = < 29 alors on borne  29
D2LBL3		INCF	FSR,f
			DECFSZ	VARK,f
			GOTO	D2LBL2			; recommencer jusqu' avoir trait 4 impulsions
FINBORN
;*************************** zones mortes et sens impulsions voies cyclique 2,3 et 4***********************
; voie 2
			MOVLW	0x10				; RAZ des masques qui ont dj subi un tri donc qui ne sont
			XORWF	ADR_RET0,w			; plus  la bonne adresse mmoire
			ADDLW	0x08
			MOVWF	FSR				; pointer MASK2A_PRV
			CLRF	INDF
			ADDLW	0x05
			MOVWF	FSR				; pointer MASK2C_PRV
			CLRF	INDF			; la voie 2 n'est jamais sur le port C
;
			MOVF	DUR_IMP2,w
			SUBLW	.75				; 75 - DUR_IMP -> w
			BTFSS	STATUS,C
			GOTO	D3LBL3
			MOVLW	0x10			; si positif DUR_IMP =< 75
			XORWF	ADR_RET0,w
			ADDLW	0x08			
			MOVWF	FSR				; pointer MASK2A_PRV
			BSF		INDF,V2M		; si DUR_IMP < 75 alors actionner V2M sur port A
			GOTO	D3FIN1
D3LBL3		MOVF	DUR_IMP2,w		; si ngatif DUR_IMP > 75
			SUBLW	.84
			BTFSC	STATUS,C
			GOTO	D3LBL4
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x08			
			MOVWF	FSR				; pointer MASK2A_PRV
			BSF		INDF,V2P			; si DUR_IMP > 84 alors appliquer sens positif V2P sur port A
			GOTO	D3FIN1
D3LBL4		MOVLW	.80
			MOVWF	DUR_IMP2			; si 76 < DUR_IMP < 84 alors 80 -> DUR_IMP
D3FIN1		
; voie 3
			MOVLW	0x10				; RAZ des masques qui ont dj subi un tri donc qui ne sont
			XORWF	ADR_RET0,w			; plus  la bonne adresse mmoire
			ADDLW	0x09
			MOVWF	FSR				; pointer MASK3A_PRV
			CLRF	INDF			; la voie 3 n'est jamais sur le portA
			ADDLW	0x05
			MOVWF	FSR				; pointer MASK3C_PRV
			CLRF	INDF
;
			MOVF	DUR_IMP3,w
			SUBLW	.75
			BTFSS	STATUS,C
			GOTO	D3LBL5
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x0E			
			MOVWF	FSR				; pointer MASK3C_PRV
			BSF		INDF,V3M		; si DUR_IMP < 75 alors actionner V3M sur port C
			GOTO	D3FIN2
D3LBL5		MOVF	DUR_IMP3,w			
			SUBLW	.84
			BTFSC	STATUS,C
			GOTO	D3LBL6
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x0E			
			MOVWF	FSR				; pointer MASK3C_PRV
			BSF		INDF,V3P			; si DUR_IMP > 84 alors appliquer sens positif V3P sur port C
			GOTO	D3FIN2
D3LBL6		MOVLW	.80
			MOVWF	DUR_IMP3			; si 76 < DUR_IMP < 84 alors 80 -> DUR_IMP
D3FIN2
; voie 4
			MOVLW	0x10				; RAZ des masques qui ont dj subi un tri donc qui ne sont
			XORWF	ADR_RET0,w			; plus  la bonne adresse mmoire
			ADDLW	0x0A
			MOVWF	FSR				; pointer MASK4A_PRV
			CLRF	INDF			; la voie 4 n'est jamais sur le portA
			ADDLW	0x05
			MOVWF	FSR				; pointer MASK4C_PRV
			CLRF	INDF
;
			MOVF	DUR_IMP4,w
			SUBLW	.75
			BTFSS	STATUS,C
			GOTO	D3LBL7
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x0F			
			MOVWF	FSR				; pointer MASK4C_PRV
			BSF		INDF,V4M		; si DUR_IMP < 75 alors actionner V4M sur port C
			GOTO	D3FIN3
D3LBL7		MOVF	DUR_IMP4,w			
			SUBLW	.84
			BTFSC	STATUS,C
			GOTO	D3LBL8
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x0F			
			MOVWF	FSR				; pointer MASK3C_PRV
			BSF		INDF,V4P			; si DUR_IMP > 84 alors appliquer sens positif V4P sur port C
			GOTO	D3FIN3
D3LBL8		MOVLW	.80
			MOVWF	DUR_IMP4			; si 76 < DUR_IMP < 84 alors 80 -> DUR_IMP
D3FIN3

; fin gestion sens
; FIN DE GESTION DES ZONES MORTES
FINZM
; DEBUT CALCULS
; TPS_ON1 (gaz) = 2,5 x(DUR_IMP1 - 29)
			MOVLW	.29
			SUBWF	DUR_IMP1,w		; DUR_IMP1 - 29 dans w
			BTFSC	STATUS,Z
			GOTO	D4LBL3			; si DUR_IMP1 = 29 moteur arrt
			MOVWF	TPS_ON1			; DUR_IMP1 - 29 dans TPS_ON1
			BCF		STATUS,C
			RRF		TPS_ON1,f		; 0.5 x (DUR_IMP1 - 29) dans TPS_ON1
			ADDWF	TPS_ON1,f		; 1.5 x(DUR_IMP1 - 29) dans TPS_ON1
			ADDWF	TPS_ON1,f		; 2.5 x(DUR_IMP1 - 29) dans TPS_ON1
			GOTO	D4LBL4
D4LBL3		CLRF	TPS_ON1			; couper moteur si DUR_IMP1 = 29
;
; TPS_ON2, 3 et 4 par adressage indirect TPS_ON = 5xABS(DUR_IMP(i) - 80)
D4LBL4		MOVLW	0x03
			MOVWF	VARK
			MOVLW	0x48
			MOVWF	FSR				; adresse DUR_IMP2 dans FSR
D4LBL2		MOVF	INDF,w
			MOVWF	DUR_IMPX		; INDF dans DUR_IMPX
			MOVLW	.80
			SUBWF	DUR_IMPX,f		; DUR_IMPX - 80 dans DUR_IMPX
			BTFSC	STATUS,C
			GOTO	D4LBL1
			COMF	DUR_IMPX,f		; ssi le rsultat de la soustraction tait ngative,
			INCF	DUR_IMPX,f		;on remet en positif
D4LBL1		MOVLW	0x07			; decalage entre DUR_IMP(i) et TPS_ON(i)
			SUBWF	FSR,f			; pointer TPS_ON(i)
			MOVF	DUR_IMPX,w
			MOVWF	INDF
			ADDWF	INDF,f
			ADDWF	INDF,f
			ADDWF	INDF,f
			ADDWF	INDF,f			; 5x|DUR_IMP(i) - 80| dans TPS_ON(i)
			MOVLW	0x08			; pointer DUR_IMP3
			ADDWF	FSR,f			; FSR - 0x0B -> FSR
;
			DECFSZ	VARK,f
			GOTO	D4LBL2			; reboucler tant que voies 2, 3 et 4 n'ont pas t traites
; discrtiser 17 incrments possibles sur les TPS_ON de 1  4 :
			MOVLW	0x04
			MOVWF	VARK
			MOVLW	0x40			; pointer TPS_ON1
			MOVWF	FSR
D4LBL7		MOVLW	.247
			SUBWF	INDF,w			; TPS_ON(i) - 247 -> w
			BTFSC	STATUS,C		
			GOTO	D4LBL8			; si TPS_ON(i) >= 247 alors 0xFF -> TPS_ON(i)
			MOVLW	0x08			; sinon ajouter 8  TPS_ON(i) et discrtiser
			ADDWF	INDF,f
			BCF		INDF,0			; sinon supprimer tous les nombres impairs
			BCF		INDF,1			; et supprimer tous les multiples de 2
			BCF		INDF,2
			BCF		INDF,3
			GOTO	D4LBL9
D4LBL8		MOVLW	0xFF
			MOVWF	INDF
D4LBL9		INCF	FSR,f
			DECFSZ	VARK,f
			GOTO	D4LBL7			; examiner TPS_ONi suivant
; fin discrtisation en 65 incrments
; FIN DE CALCUL DES TPS_ON
FINCALC
; DEBUT TRI DES TPS_ON(i) et des MASK(i)********************************DETAIL 5
; initialisation MASKA_PRV 
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x06
			MOVWF	FSR
			CLRF	INDF				; pointer 0-> MASK0A_PRV
			INCF	FSR,f
			CLRF	INDF			; pointer 0-> MASK1A_PRV (les gaz ne sont jamais sur le port A)
;
; initialisation MASKC_PRV
			MOVLW	0x04			
			ADDWF	FSR,f			; pointer MASK0C_PRV
			MOVF	PORTC,w			; lire le port pour connaitre tat de la led
			ANDLW	0x08
			MOVWF	INDF			; tat de la led seule -> MASK0C_PRV
			INCF	FSR,f			; pointer MASK1C_PRV
			MOVLW	b'00000001'		; init le bit des gaz
			MOVWF	INDF
; fin init masques
			MOVLW	0x07
			MOVWF	VARN			; ne peut pas tre VARI cause interruption
D5LBL1		MOVLW	0x40			; adresse de TPS_ON1
			MOVWF	VARK
; tri des TPS_ON
D5LBL2		MOVF	VARK,w
			MOVWF	FSR				; pointe TPS_ON(i)
			INCF	FSR,f			; pointe TPS_ON(i+1)
			MOVF	INDF,w			; TPS_ON(i+1) dans w
			DECF	FSR,f
			SUBWF	INDF,w			; TPS_ON(i) - TPS_on(i+1) dans w
			BTFSC	STATUS,C		; TPS_ON(i+1) > TPS_on(i) ?
			GOTO	D5LBL3			; non
			CALL	PERMUT			; si oui permuter les valeurs via TPS_ONX
; tri des MASKA
			MOVLW	0x10			; dcalage de TPS_ON (i+1)  MASK(i)A_PRV
			XORWF	ADR_RET0,w
			SUBLW	0x3A
			SUBWF	FSR,f			; FSR - (0x3A - ADR_RET0_PRV) -> FSR
			CALL 	PERMUT			; permuter les masques du port A
; tri des MASKC
			MOVLW	0x04			; dcalage de MASK(i+1)A_PRV  MASK(i)C_PRV
			ADDWF	FSR,f
			CALL 	PERMUT			; permuter les masques du port C
;
D5LBL3		INCF	VARK,f
			MOVF	VARK,w
			SUBLW	0x43			; adresse de tps_on4
			BTFSC	STATUS,C		; a-t-on fait les 3 tests (comparaisons 2  2)?
			GOTO	D5LBL2			; si non, test suivant
			DECFSZ	VARN,f			; si oui, a-t-on fait 6 balayages ?
			GOTO	D5LBL1			; si non, balayage suivant
; FIN DU TRI ************************************************************************************
FINTRI
; CALCUL DES RETARDS PROVISOIRES*********************************************************DETAIL 6
; RET0_PRV = 255 - TPS_ON1
; RET1_PRV = TPS_ON1 - TPS_ON2
; RET(i)_PRV = TPS_ON(i) - TPS_ON(i+1)
; RET4_PRV = TPS_ON4 - TPS_ON5 avec TPS_ON5 = 0
;
; calcul de RET0
			MOVLW	0x10
			XORWF	ADR_RET0,w
			MOVWF	FSR				; adresse(RET0_PRV) -> FSR
			MOVF	TPS_ON1,w
			SUBLW	0xFF			; 255 - TPS_ON(i) -> RET0_PRV	
			MOVWF	INDF			
; calcul des autres retards, cas gnral
			MOVLW	0x40			; adresse de TPS_ON1
			MOVWF	ADR	
D6LBL1		MOVF	ADR,w			; adresse de TPS_ON(i) avec i = nb de passage de 1  4
			MOVWF	FSR
			INCF	FSR,f
			MOVF	INDF,w			; TPS_ON(i+1) dans w
			DECF	FSR,f
			SUBWF	INDF,w			; TPS_ON(i) - TPS_ON(i+1) dans w
			MOVWF	TPS_ONX			; stockage TPS_ON(i) - TPS_ON(i+1) dans TPS_ONX
			MOVLW	0x10			; calculer dcalage de TPS_ON(i)  RET(i)_PRV
			XORWF	ADR_RET0,w
			SUBLW	0x3F
			SUBWF	FSR,f			; FSR - (0x3F - ADR_RET0_PRV) -> FSR = adresse RET(i)_PRV
			MOVF	TPS_ONX,w
			MOVWF	INDF			; TPS_ONX dans RET(i)_PRV
;
			INCF	ADR,f
			MOVF	ADR,w
			SUBLW	0x43
			BTFSC	STATUS,C		; a-t-on dj trait TPS_ON4 ?
			GOTO	D6LBL1			; si non, reboucler jusqu'
									; si oui, rien
; recherche des retards nuls, dcalage et fusion des masques inutiles
RECH_NUL	MOVLW	0x10
			XORWF	ADR_RET0,w
			MOVWF	ADR				; adresse de RET0_PRV -> ADR
			MOVLW	0x05
			MOVWF	VARN			; init le compteur de boucles
D9LBL1		MOVF	ADR,w
			MOVWF	FSR
			MOVF	INDF,w
			BTFSS	STATUS,Z
			GOTO	D9LBL3
			CALL	FUSION			; si RET_PRV(ADR) = 0 dcaler les retards et fusionner les masques
			DECFSZ	VARN,f			; a-t-on trait tous les RET_PRV ?
			GOTO	D9LBL1			; non on reboucle
			GOTO	LBL5			; si oui alors fin
D9LBL3		INCF	ADR,f			; continuer le balayage
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x04			; adresse de RET4_PRV -> w
			SUBWF	ADR,w			; ADR - adresse de RET4_PRV -> w
			BTFSS	STATUS,Z		; a-t-on fini de balayer ?
			GOTO	D9LBL1			; si non reboucler
								; si oui passer  la suite						
; FIN DU CALCUL DES RETARDS PROVISOIRES***********************************************************
; GESTION DE LA FIN DE LA PAUSE DE 8.5 ms
LBL5		BTFSS	PORTA,ENTIMP
			GOTO	LBL6			; signal entre  zro trop tt, pause trop courte
;			BTFSC	PIR1,TMR1IF
;			CALL	DEB_T1
			BTFSS	INTCON,T0IF
			GOTO	LBL5			; tout est normal, on attend la fin TMR0 (entre  1 obligatoire)
			BCF		INTCON,T0IF		; TMR0 overflow, la pause a t respecte
			CLRF	CPT_TI
			MOVLW	.99
			MOVWF	TMR1H			; et on RAZ le dfaut de signal absent
			CLRF	TMR1L			; idem
			MOVF	CPT_DEM,w
			BTFSS	STATUS,Z		; CPT_DEM = 0 ?
			GOTO	LBL7			; si non, tester le manche des gaz pour scurit
			BSF		FLAGS,FNT		; si oui on signale qu'une nouvelle trame est dispo
;
;			BCF		PIR1,TMR2IF
;			BANK1
;			MOVLW	b'00000010'		; et on autorise le signal PWM
;			MOVWF	PIE1			; TMR2 int enable
;			MOVF	PR2,w			; si PR2 est  zro on risque le bug
;			BTFSS	STATUS,Z
;			GOTO	LBL10
;			MOVLW	0xFF
;			MOVWF	PR2				; alors on rinitalise PR2
;			BANK0
;			MOVWF	SUMRET			; et on rinitialise le gnrateur PWM
;			MOVLW	0x05
;			MOVWF	VARI
LBL10		GOTO	NEXTTRAM		; trame OK mise  dispo, on peut acqurir la suivante
LBL7		MOVLW	.31
			SUBWF	DUR_IMP1,w		; DUR_IMP1 - 31 dans w
			BTFSC	STATUS,C		; les gaz sont-ils  zero ?
			GOTO	LBL8			; non
			DECF	CPT_DEM,f		; oui, dcompte du temps de scurit
			GOTO	NEXTTRAM		; et attente trame suivante
LBL8		MOVLW	0x02			; si gaz pas  zro,
			MOVWF	CPT_TI			; recharger CPT_TI pour faire clignoter la led
			GOTO	NEXTTRAM		; et attendre que les gaz soient  zro
LBL6		CALL	GERER_DEF		; la pause a t trop courte, on ne latche pas les retards
;									 et on gre les dfauts
			GOTO	ATTPAUSE		; dans les 2 cas attendre impulsion suivante
; FIN BOUCLE PRINCIPALE
; 
; sub attendre front montant
ATT_FM		BTFSS	PORTA,ENTIMP
			GOTO	LBL3
			BTFSS	PIR1,TMR1IF
			GOTO	ATT_FM
			CALL	DEB_T1			; signaler un dfaut de rception grave (dbordement TMR1)
			GOTO	ATT_FM
LBL3		BTFSS	PORTA,ENTIMP
			GOTO	LBL3
LBL9		RETLW	0x00
; sub permuter le contenu des adresses FSR et FSR+1
PERMUT		MOVF	INDF,w
			MOVWF	TPS_ONX		; TPS_ON(i)	dans TPS_ONX
			INCF	FSR,f
			MOVF	INDF,w			; TPS_ON(i+1) dans w
			DECF	FSR,f
			MOVWF	INDF			; TPS_ON(i+1) dans TPS_ON(i)
			INCF	FSR,f
			MOVF	TPS_ONX,w
			MOVWF	INDF			; TPS_ONX dans TPS_ON(i+1)
			RETLW	0x00
; sub tout au neutre
TOUTNEUTRE	
;			BANK1
;			BCF		PIE1,TMR2IE		; on dsactive le signal PWM : TMR2 int disable
;			BANK0
;			BCF		PORTA,V2M		; couper la sortie voie 2
;			BCF		PORTA,V2P		; couper la sortie voie 2
;			MOVLW	b'00001000'		; RAZ PORTC mais laisser la led dans l'tat
;			ANDWF	PORTC,f
;			MOVLW	b'00001000'
;			XORWF	PORTC,f			; toggle sortie LED, afficher le dfaut
;			MOVLW	0x05
;			MOVWF	VARI			; faire croire au gnrateur PWM qu'il est en bout de cycle -> sorties  zro
;			CLRF	CPT_TI			; pour permettre clignotement led
;			CLRF	MASK0A_A
;			BSF		MASK0A_A,LED	; teindre la led
;			CLRF	MASK0A_B
;			BSF		MASK0A_B,LED	; teindre la led
			GOTO	HOT_RESET
			RETLW	0x00
; sub latchage des masques et retards
LATCH		MOVLW	0x10
			XORWF	ADR_RET0,f		; si oui, on latche les retards
;
			BCF		FLAGS,FNT		; et on signale que les valeurs sont primes
 			RETLW	0x00
; sub gestion de l'affichage des dfauts
GERER_DEF	INCF	CPT_TI,f		; la pause a t trop courte, on ne latche pas les retards
;									 et on compte les dfauts
			BTFSS	CPT_TI,2
			GOTO	G1
			BSF		MASK0C_A,LED	; changer l'tat de la led 1 passage sur 4
			BSF		MASK0C_B,LED
			GOTO	G2
G1			BCF		MASK0C_A,LED	; changer l'tat de la led 1 passage sur 4
			BCF		MASK0C_B,LED
G2			MOVF	CPT_TI,w
			SUBLW	.25
			BTFSS	STATUS,C		; CPT_TI > 25 ? (pour 1s de perte de controle)
			CALL	TOUTNEUTRE		; si oui, on a perdu le signal, tout au neutre
			RETLW	0x00
; sub DECALE : dcale  l'adresse -1 les donnes comprises entre l'adresse VARK et l'adresse ADR_FIN			
DECALE		INCF	VARK,w
			MOVWF	FSR				; VARK + 1 -> FSR
			MOVF	INDF,w			; contenu de adresse (VARK+1) -> w
			DECF	FSR,f			; VAK -> FSR
			MOVWF	INDF			; contenu de adresse (VARK+1) -> contenu de adresse(VARK)
			INCF	VARK,f
			MOVF	VARK,w			; VARK + 1 -> w
			SUBWF	ADR_FIN,w		; ADR_FIN - (VARK+1) -> w
			BTFSS	STATUS,Z		; VARK+1 = ADR_FIN ?
			GOTO	DECALE			; si non on continue  balayer
			RETLW	0x00			; si oui alors fin
; sub FUSION : fusionne les masques A et C indice i+1 et i vers i (ADR contient l'adresse de RET_PRV(i)
FUSION		MOVF	ADR,w
			ADDLW	0x07
			MOVWF	FSR				; 0x07 + (ADR) -> FSR
			MOVF	INDF,w			; MASK(i+1)A_PRV -> w
			DECF	FSR,f			; pointer MASK(i)A_PRV
			IORWF	INDF,f			; MASK(i+1)A_PRV OU MASK(i)A_PRV -> MASK(i)A_PRV
;
			MOVLW	0x06			; dcalage entre MASK(i)A_PRV et MASK(i)C_PRV
			ADDWF	FSR,f
			MOVF	INDF,w
			DECF	FSR,f
			IORWF	INDF,f			; MASK(i+1)C_PRV AND MASK(i)C_PRV -> MASK(i)C_PRV
; dcaler les RET_PRV de i (ADR) jusqu' RET4_PRV
			MOVF	ADR,w
			MOVWF	VARK
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x05			; adresse (RET5_PRV) (OK)
			MOVWF	ADR_FIN
			CALL 	DECALE
; dcalage des masques uniquement si i+2 =< 4 donc si i =< 2
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x03
			SUBWF	ADR,w			; ADR-Adresse(RET3_PRV) ->w
			BTFSC	STATUS,C		; rsultat ngatif ?
			RETLW	0x00			; si oui shunter le dcalage des masques si i>2
; si non dcaler les MASKA_PRV de i+2 vers i+1 jusqu' MASK4A_PRV ssi 
			MOVF	ADR,w
			ADDLW	0x07
			MOVWF	VARK			; adresse MASK(i+1)A_PRV -> VARK
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x0A			; adresse (MASK4A_PRV)
			MOVWF	ADR_FIN
			CALL 	DECALE
; et dcaler les MASKC_PRV de i+2 vers i+1 jusqu' MASK4C_PRV
			MOVF	ADR,w
			ADDLW	0x0C
			MOVWF	VARK			; adresse MASK(i+1)C_PRV -> VARK
			MOVLW	0x10
			XORWF	ADR_RET0,w
			ADDLW	0x0F			; adresse (MASK4C_PRV)
			MOVWF	ADR_FIN
			CALL 	DECALE
			RETLW	0x00
; sub gestion d'un dfaut de rception de 2s (dbordement du TMR1)
DEB_T1		BCF		PIR1,TMR1IF
			MOVLW	.99
			MOVWF	TMR1H			; Rechargement tmr1
			CLRF	TMR1L			; idem
			CALL	GERER_DEF
			RETLW	0x00
;
; initialize eeprom locations
;
;EE			CODE	0x2100
;			DE	0x00, 0x01, 0x02, 0x03
;
			END
